import numpy as np
import matplotlib.pyplot as plt
import cv2 as cv
from tqdm import tqdm
from glob import glob

from sklearn.cluster import KMeans

def centroid_histogram(clt):
	numLabels = np.arange(0, len(np.unique(clt.labels_)) + 1)
	(hist, _) = np.histogram(clt.labels_, bins = numLabels)

	hist = hist.astype("float")
	hist /= hist.sum()

	return hist

def plot_colors(hist, centroids):

	bar = np.zeros((50, 300, 3), dtype = "uint8")
	startX = 0

	for (percent, color) in zip(hist, centroids):

		endX = startX + (percent * 300)
		cv.rectangle(bar, (int(startX), 0), (int(endX), 50),
			color.astype("uint8").tolist(), -1)
		startX = endX
	
	return bar

filenames = glob('./img_beverage/*')

for filename in tqdm(filenames[16:]):
    img = cv.imread(filename)
    image2 = cv.cvtColor(img, cv.COLOR_BGR2RGB)
    clusters = 2
    
    '''plt.figure()
    plt.axis("off")
    plt.imshow(image2)'''
    
    image = image2.reshape((image2.shape[0] * image2.shape[1], 3))
    clt = KMeans(n_clusters = clusters)
    clt.fit(image)
    

    
    clustered = clt.labels_.reshape(img.shape[:-1])
    clustered = clt.cluster_centers_[clustered].astype("uint8")
    
    hist = centroid_histogram(clt)
    bar = plot_colors(hist, clt.cluster_centers_)
    
    plt.figure()
    plt.axis("off")
    plt.subplot(121),plt.imshow(image2,cmap = 'gray')
    plt.title('Original Image'), plt.xticks([]), plt.yticks([])
    plt.subplot(122),plt.imshow(clustered,cmap = 'gray')
    plt.title('Clustered Image'), plt.xticks([]), plt.yticks([])
    #plt.imshow(bar)
    plt.show()
    
    img = cv.cvtColor(img, cv.COLOR_RGB2GRAY)
    edges = cv.Canny(img,100,200)
    
    plt.subplot(121),plt.imshow(img,cmap = 'gray')
    plt.title('Original Image'), plt.xticks([]), plt.yticks([])
    plt.subplot(122),plt.imshow(edges,cmap = 'gray')
    plt.title('Edge Image'), plt.xticks([]), plt.yticks([])
    plt.show()
    
resized_img = cv.resize(edges, (300,400), interpolation = cv.INTER_AREA)
plt.figure()
plt.axis("off")
plt.imshow(resized_img)