import numpy as np
import matplotlib.pyplot as plt
import cv2 as cv
from tqdm import tqdm
from glob import glob

from sklearn.cluster import KMeans

def centroid_histogram(clt):
	numLabels = np.arange(0, len(np.unique(clt.labels_)) + 1)
	(hist, _) = np.histogram(clt.labels_, bins = numLabels)

	hist = hist.astype("float")
	hist /= hist.sum()

	return hist

def plot_colors(hist, centroids):

	bar = np.zeros((50, 300, 3), dtype = "uint8")
	startX = 0

	for (percent, color) in zip(hist, centroids):

		endX = startX + (percent * 300)
		cv.rectangle(bar, (int(startX), 0), (int(endX), 50),
			color.astype("uint8").tolist(), -1)
		startX = endX
	
	return bar

filenames = glob('./img_beverage/*')

#img = cv.imread(filenames[3])
for filename in tqdm(filenames):
    img = cv.imread(filename)
    image2 = cv.cvtColor(img, cv.COLOR_BGR2RGB)
    clusters = 2
    
    '''plt.figure()
    plt.axis("off")
    plt.imshow(image2)'''
    
    image = image2.reshape((image2.shape[0] * image2.shape[1], 3))
    clt = KMeans(n_clusters = clusters)
    clt.fit(image)
    
    
    
    clustered = clt.labels_.reshape(img.shape[:-1])
    clustered = clt.cluster_centers_[clustered].astype("uint8")
    
    hist = centroid_histogram(clt)
    bar = plot_colors(hist, clt.cluster_centers_)
    
    plt.figure()
    plt.axis("off")
    plt.subplot(121),plt.imshow(image2,cmap = 'gray')
    plt.title('Original Image'), plt.xticks([]), plt.yticks([])
    plt.subplot(122),plt.imshow(clustered,cmap = 'gray')
    plt.title('Clustered Image'), plt.xticks([]), plt.yticks([])
    #plt.imshow(bar)
    plt.show()
    
    img = cv.cvtColor(img, cv.COLOR_RGB2GRAY)
    edges = cv.Canny(img,100,200)
    
    '''plt.subplot(121),plt.imshow(img,cmap = 'gray')
    plt.title('Original Image'), plt.xticks([]), plt.yticks([])
    plt.subplot(122),plt.imshow(edges,cmap = 'gray')
    plt.title('Edge Image'), plt.xticks([]), plt.yticks([])
    plt.show()'''
        
    x,y = 0,0
    if len(edges)>len(edges[0]):
        x=(len(edges)-len(edges[0]))/2
    if len(edges)<len(edges[0]):
        y=(len(edges[0])-len(edges))/2
    x,y = np.int32(x),np.int32(y)
    resized_img = cv.copyMakeBorder(edges,y,y,x,x,cv.BORDER_CONSTANT,value=[0,0,0])
    
    import random as rng
    
    # Draw contours + hull results
    '''drawing = np.zeros((resized_img.shape[0], resized_img.shape[1], 3), dtype=np.uint8)
    for i in range(len(contours)):
        color = (rng.randint(0,256), rng.randint(0,256), rng.randint(0,256))
        cv.drawContours(drawing, contours, i, color)
        cv.drawContours(drawing, hull_list, i, color)
    cv.fillPoly(drawing, pts =hull_list, color=(255,255,255))'''
    drawing = cv.dilate(resized_img, np.ones((5,5),np.uint8), iterations=7)
    drawing = cv.erode(drawing, np.ones((7,7),np.uint8), iterations=3)
    
    contours, _ = cv.findContours(drawing, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
    # Find the convex hull object for each contour
    hull_list = []
    hull_all = []
    for i in range(len(contours)):
        hull = cv.convexHull(contours[i])
        hull_list.append(hull)
        hull_all.append(contours[i])
    
    drawing2 = np.zeros((resized_img.shape[0], resized_img.shape[1], 3), dtype=np.uint8)
    for i in range(len(contours)):
        color = (rng.randint(0,256), rng.randint(0,256), rng.randint(0,256))
        cv.drawContours(drawing2, contours, i, color)
        cv.drawContours(drawing2, hull_list, i, color)
    cv.fillPoly(drawing2, pts =hull_list, color=(255,255,255))
    drawing2 = cv.morphologyEx(drawing2, cv.MORPH_CLOSE, (7,7))
    # Show in a window
    '''plt.figure()
    plt.axis("off")
    plt.imshow(drawing,cmap = 'gray')
    
    plt.figure()
    plt.axis("off")
    plt.imshow(resized_img,cmap = 'gray')'''
    
    plt.subplot(131),plt.imshow(edges,cmap = 'gray')
    plt.title('Edge Image'), plt.xticks([]), plt.yticks([])
    plt.subplot(132),plt.imshow(drawing,cmap = 'gray')
    plt.title('Hull Image'), plt.xticks([]), plt.yticks([])
    plt.subplot(133),plt.imshow(drawing2,cmap = 'gray')
    plt.title('Filled Image'), plt.xticks([]), plt.yticks([])
    plt.show()
    
    #cv.imwrite('./extended/img_beauty/edge/'+filename.split('\\')[1], edges)
    #cv.imwrite('./extended/img_beauty/filled/'+filename.split('\\')[1], drawing)