import pandas as pd
from operator import itemgetter
from copy import deepcopy
import numpy as np


def read_rules(category, criterion):
    rules_file = open('./rules/'+category+'_'+criterion+'.rules',)
    text = rules_file.read()
    rules_file.close()
    return text

def format_rules(rule_to_format):
    format_rule = rule_to_format.split(':')
    format_rule[1] = format_rule[1].replace('(','').replace(')','')
    rule_conditions = format_rule[1].split('|')[0].split('=>')[0].replace(' ','').replace('_II','').replace('_I','').replace('>=',' \\geq ').replace('<=',' \\leq ').split('&')
    rule_class = format_rule[1].split('|')[0].split('=>')[1].replace(' ','').replace('>=',' \\geq ').replace('<=',' \\leq ').replace('pct','class')
    rule_confidence = float(format_rule[6].split('\n')[0])
    rule_coverage = int(format_rule[8].split('\n')[0])
    rule_coverage_factor = round(float(format_rule[7].split('\n')[0]), 2)
    rule = {'conditions':rule_conditions, 'class':rule_class, 'confidence':rule_confidence, 'coverage': rule_coverage, 'coverage_factor':rule_coverage_factor}
    return rule

#Do wymienienia w tabeli
def rule_to_latex(rule):
    rule_string = 'Jeżeli $'+rule['conditions'][0]
    if len(rule['conditions'])>1:
        for cond in rule['conditions'][1:]:
            rule_string+=' \\wedge '+cond
    rule_string+='$ to $'+rule['class']+'$ & '+str(rule['confidence'])+' & '+str(rule['coverage'])+' & '+str(rule['coverage_factor'])+' \\\\ \\hline'
    return rule_string

#Do wymienienia w itemize
def rule_to_appendix(rule):
    rule_string = '\item Jeżeli $'+rule['conditions'][0]
    if len(rule['conditions'])>1:
        for cond in rule['conditions'][1:]:
            rule_string+=' \\wedge '+cond
    rule_string+='$ to $'+rule['class']+'$, Trafność: '+str(rule['confidence'])+', Pokryte przykłady: '+str(rule['coverage'])+', stopień pokrycia unii klas: '+str(rule['coverage_factor'])
    return rule_string

dataset = pd.read_csv('./beauty_extended.csv', sep=';', index_col='Brand')

decision = dataset.columns.values[:-9]

rules_count = {'upper': {'2.0':0, '3.0':0, '4.0':0, '5.0':0},
               'lower': {'1.0':0, '2.0':0, '3.0':0, '4.0':0},
               'upper_count':0, 'lower_count':0}
attributes_labels = ['H','Shape','V', 'Symmetry_Horizontal','Symmetry_Vertical','Edge', 'S', 'Mark_or_text']
conditions_count = {'Lengths':[], 'Coverages':[]}
for att_label in attributes_labels:
    conditions_count[att_label] = 0

chapter4 = ''
appendix = {'beauty':'','beverage':''}
stats = {'class_info':deepcopy(rules_count), 'cond_info':deepcopy(conditions_count)}
for crit in decision:
    beauty_rules = []
    beverage_rules = []
    stats[crit] = dict()
    chapter4+='\n'+crit.replace('_','\\_')+':\n'
    for category in ['beauty', 'beverage']:
        rules = read_rules(category, crit).split('[RULESINFO]')[1]
        rules_atleast = rules.split('Certain at least rules')[1].split('#Certain at most rules')[0].split('\n\n')[:-1]
        rules_atmost = rules.split('#Certain at most rules')[1].split('\n\n')[:-1]
        
        formated_rules = []
        for rule_to_format in rules_atleast:
            if category == 'beauty': beauty_rules.append(format_rules(rule_to_format))
            else: beverage_rules.append(format_rules(rule_to_format))
            formated_rules.append(format_rules(rule_to_format))
        for rule_to_format in rules_atmost:
            if category == 'beauty': beauty_rules.append(format_rules(rule_to_format))
            else: beverage_rules.append(format_rules(rule_to_format))
            formated_rules.append(format_rules(rule_to_format))
            
        appendix[category]+='\n\\subsection*{'+crit.replace('_pct', '').replace('_','\\_')+'}\n\\begin{enumerate}\n'
        for fr in formated_rules:
            appendix[category]+='    '+rule_to_appendix(fr).replace('_','\\_')+'\n'
        appendix[category]+='\\end{enumerate}\n'
        
        if category == 'beauty': beauty_rules = sorted(beauty_rules, key=itemgetter('coverage_factor', 'confidence'), reverse=True)
        else: beverage_rules = sorted(beverage_rules, key=itemgetter('coverage_factor', 'confidence'), reverse=True)
            
        chapter4+='\n'+category+'\n'
        if category == 'beauty':
            for fr in beauty_rules:
                chapter4+=rule_to_latex(fr).replace('_','\\_')+'\n'
        else:
            for fr in beverage_rules:
                chapter4+=rule_to_latex(fr).replace('_','\\_')+'\n'
             
        stats[crit][category] = {'class_info':deepcopy(rules_count), 'cond_info':deepcopy(conditions_count)}
        for fr in formated_rules:
            rule_type = 'upper' if 'geq' in fr['class'] else 'lower'
            stats[crit][category]['class_info'][rule_type+'_count']+=1
            stats[crit][category]['class_info'][rule_type][fr['class'].split('eq ')[1]]+=1
            stats[crit][category]['cond_info']['Lengths'].append(len(fr['conditions']))
            stats[crit][category]['cond_info']['Coverages'].append(fr['coverage'])
            stats['class_info'][rule_type+'_count']+=1
            stats['class_info'][rule_type][fr['class'].split('eq ')[1]]+=1
            stats['cond_info']['Lengths'].append(len(fr['conditions']))
            stats['cond_info']['Coverages'].append(fr['coverage'])
            for cond in fr['conditions']:
                attribute = 'Mark_or_text'
                for att in attributes_labels:
                    if cond.startswith(att):
                        attribute = att
                        break
                stats[crit][category]['cond_info'][attribute]+=1
                stats['cond_info'][attribute]+=1

stats_string = {'beauty':'', 'beverage':''}
for category in ['beauty', 'beverage']:
    for crit in decision:
        stats_string[category]+= crit.replace('_pct', '').replace('_','\_') + ' & '
        stats_string[category]+= str(stats[crit][category]['class_info']['lower_count']+stats[crit][category]['class_info']['upper_count']) + ' & '
        stats_string[category]+= str(stats[crit][category]['class_info']['upper_count'])+' & '+str(stats[crit][category]['class_info']['lower_count']) + ' & '
        stats_string[category]+= str(min(stats[crit][category]['cond_info']['Lengths']))+' - '+str(max(stats[crit][category]['cond_info']['Lengths'])) + ' & '
        stats_string[category]+= str(round(np.mean(stats[crit][category]['cond_info']['Lengths']), 2)) + ' & '
        stats_string[category]+= str(min(stats[crit][category]['cond_info']['Coverages']))+' - '+str(max(stats[crit][category]['cond_info']['Coverages'])) + ' & '
        stats_string[category]+= str(round(np.mean(stats[crit][category]['cond_info']['Coverages']), 2)) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['H']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['S']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['V']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['Shape']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['Symmetry_Horizontal']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['Symmetry_Vertical']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['Edge']) + ' & '
        stats_string[category]+= str(stats[crit][category]['cond_info']['Mark_or_text'])
        
        
        stats_string[category]+= ' \\\\ \\hline\n'